#ifndef _VISIBLEFORM_
#define _VISIBLEFORM_

#include "MainForm.h"
#include "all.h"
#include "MathObject.h"
#include "OGLWidgetMain.h"
#include "OGLWidgetFloating.h"
#include "OGLWidgetFloating2.h"
#include "structs.h"
#include "InstructionBrowser.h"

//qt includes
#include <qapplication.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdial.h>
#include <qdir.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qimage.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qtimer.h>
#include <qtoolbox.h>

class MathObject;

class VisibleForm : public MainForm {
  public:
    VisibleForm ( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~VisibleForm();

  private:
    OGLWidgetMain		*vp;		// Viewing Pane
    OGLWidgetFloating	*fwin_vp; // Floating Window Viewing Pane
    OGLWidgetFloating2	*fwin2_vp;

  public:
    MathObject		*mo;		// Math Object
    HelpWindow		*help;
    int				swapFrames;	//
    int				swapFrames2;

  protected:
    void			initForm			( void );
    void			resizeEvent			( QResizeEvent* );		
    void			closeEvent			( QCloseEvent* );
    void			genHelpString		( void );
    QString			helpstr;
	bool			eventFilter			( QObject *target, QEvent *event );
	void			timerEvent			( QTimerEvent* );
  public:
    void			exitApp				( void );
    void			notifyOGLWidgets	( void );
    void            notifyOGLFloatingWidget ( void );
    void			calbak_Help			( void );
    void			calbak_Clear		( void );
	void			calbak_OnScreenText	( void );
    void			stateChange			( void );
    void			updateForm			( void );
	void			updateObject		( void );
	void			updateQuaternionRingElements ( void );

  public:
	//----------------------------------------------//
	/* ADMINISTRATION */
	//----------------------------------------------//

    virtual void calbak_ResetApp			( void );
    virtual void calbak_ViewControl_Axes	( );
    virtual void calbak_ViewControl_Faces	( );
    virtual void calbak_ViewControl_Frames	( );
    virtual void calbak_ViewControl_EdgesOnly ( );
    virtual void calbak_ViewControl_Labels	( );
    virtual void calbak_ViewControl_Palette	( );
    virtual void calbak_ViewControl_StatusBar       ( );
    virtual void calbak_DumpScreen			        ( );
    virtual void calbak_DrawingOptions		        ( );
	virtual void calbak_DrawingOptions_Palette      ( );
    virtual void calbak_DrawingOptions_CurveType    ( );
    virtual void calbak_DrawingOptions_CurveType_SplineType ( );
    virtual void calbak_Swap_Frames			( void );
    virtual void calbak_Swap_Frames_2		( void );
            
	//----------------------------------------------//
	/* KNOT */
	//----------------------------------------------//
    virtual void calbak_KnotTransform_Param_m			( void ); //
    virtual void calbak_KnotTransform_Param_n			( void ); //
    virtual void calbak_KnotTransform_Param_a			( void ); //
    virtual void calbak_KnotTransform_Param_b			( void ); //
    virtual void calbak_KnotTransform_Param_r			( void ); //
    virtual void calbak_KnotTransform_Param_startangle	( void ); //
    virtual void calbak_KnotTransform_Param_startangle_2( void ); //
    virtual void calbak_KnotTransform_LockAB			( void );
    virtual void calbak_KnotTransform_Frames            ( void );
    virtual void calbak_KnotTransform_QuatMapFrames     ( );
    virtual void calbak_KnotTransform_QuatFrames        ( void );
    virtual void calbak_KnotControl_Gap					( );
    virtual void calbak_KnotControl_Curve				( );
    virtual void calbak_KnotControl_GoTwice				( );
    virtual void calbak_KnotControl_EditQuatMap			( );
    virtual void calbak_KnotTransform_QuatMap_ShowSphere( );
    virtual void calbak_KnotTransform_QuatMap_Rings     ( );
	virtual void calbak_KnotTransform_QuatMap_FullSpace_Mesh( );
	virtual void calbak_KnotTransform_QuatMap_Space_Surface ( );
    virtual void calbak_KnotTransform_QuatComponent			( );
    virtual void calbak_QuatRings_Number					( );
    virtual void calbak_TrackRing							( );
	virtual void calbak_QuatRings_AngleChange				( void );
    virtual void calbak_ViewControl_QMap_TubeThickness		( void );
    virtual void calbak_ViewControl_3DCurve_TubeThickness	( void );


	//----------------------------------------------//
	/* CURVE*/
	//----------------------------------------------//
    virtual void calbak_SplineTransform_t		( );

	//----------------------------------------------//
	/* SURFACE */
	//----------------------------------------------//
    virtual void calbak_Surface_Param_U			( void );
    virtual void calbak_Surface_Param_V			( void );
    virtual void calbak_Surface_Param_A			( void );
    virtual void calbak_Surface_Param_B			( void );
	virtual void calbak_Surface_Param_R			( void );
    virtual void calbak_Surface_SelectType		( void );

	//----------------------------------------------//
	/* OPTIMIZATIONS */
	//----------------------------------------------//
	virtual void calbak_Optimization_Start		( void );
	virtual void calbak_Optimization_Stop		( void );
	virtual void calbak_Optimization_Update		( void );
	virtual void calbak_Optimization_Increment_Inc	( void );
	virtual void calbak_Optimization_Increment_Dec	( void );
	virtual void calbak_Optimization_SingleStep	( void );

	//----------------------------------------------//
	/* GRAPH */
	//----------------------------------------------//
    virtual void calbak_Graph_cbox_Qdist		( void );
    virtual void calbak_Graph_cbox_Qdotprod		( void );
    virtual void calbak_Graph_cbox_Qtangacc		( void );
    virtual void calbak_Graph_cbox_Qcurvature	( void );
    virtual void calbak_Graph_cbox_Qtorsion		( void );

	//----------------------------------------------//
	/* TEXTURE MGMT */
	//----------------------------------------------//
	void		 texUpdateList					( void );
	virtual void calbak_Texture_Load			( const QString& text );
	virtual void calbak_Texture_ControlParam	( void );
	virtual void calbak_Texture_Toggle			( void );
    virtual void calbak_DrawingOptions_ColorCode( void );

    /* LOADING MESHVIEW FILES */
   	void		 curvesUpdateList				( void );
    virtual void calbak_CurveList_Load( const QString & text );

};

#endif
